# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from game.end_to_end_tests.base_game_test import BaseGameTest
from game.models import Level, Block, LevelBlock

from game.theme import get_theme
from game.character import get_character


class TestCowCrashes(BaseGameTest):
    cow_level = None

    def test_crash_into_cow_going_forward(self):
        self.run_cow_crashing_test("crash_into_cow_going_forward")

    def test_crash_into_cow_turning_left(self):
        self.run_cow_crashing_test("crash_into_cow_turning_left")

    def test_crash_into_cow_turning_right(self):
        self.run_cow_crashing_test("crash_into_cow_turning_right")

    def test_crash_into_cow_on_t_junction(self):
        self.run_cow_crashing_test("crash_into_cow_on_t_junction")

    def test_crash_into_cow_on_t_junction2(self):
        self.run_cow_crashing_test("crash_into_cow_on_t_junction2")

    def test_crash_into_cow_on_t_junction3(self):
        self.run_cow_crashing_test("crash_into_cow_on_t_junction3")

    def test_crash_into_cow_on_crossroads_junction(self):
        self.run_cow_crashing_test("crash_into_cow_on_crossroads_junction")

    def run_cow_crashing_test(self, workspace_file):
        user_profile = self.login_once()
        TestCowCrashes.cow_level.owner = user_profile
        TestCowCrashes.cow_level.save()
        workspace_id = self.use_workspace(workspace_file, user_profile)
        self.go_to_custom_level(TestCowCrashes.cow_level) \
            .load_solution(workspace_id) \
            .run_cow_crashing_program()

    @classmethod
    def setUpClass(cls):
        BaseGameTest.setUpClass()
        grass = get_theme(name='grass')

        van = get_character('Van')

        TestCowCrashes.cow_level = Level(name='Cow crashing',
                                         anonymous=False,
                                         blocklyEnabled=True,
                                         character=van,
                                         cows='[{"minCows":"7","maxCows":"7","potentialCoordinates":[{"x":4,"y":4},{"x":2,"y":4},{"x":3,"y":7},{"x":4,"y":6},{"x":2,"y":6},{"x":3,"y":1},{"x":4,"y":2}],"type":"WHITE"}]',
                                         default=False,
                                         destinations='[[4,5]]',
                                         direct_drive=True,
                                         fuel_gauge=False,
                                         max_fuel=50,
                                         model_solution='[1]',
                                         origin='{"coordinate":[2,5],"direction":"E"}',
                                         path='[{"coordinate":[2,5],"connectedNodes":[1]},{"coordinate":[3,5],"connectedNodes":[0,4,2,5]},{"coordinate":[4,5],"connectedNodes":[1]},{"coordinate":[3,7],"connectedNodes":[4]},{"coordinate":[3,6],"connectedNodes":[8,3,6,1]},{"coordinate":[3,4],"connectedNodes":[10,1,11,16]},{"coordinate":[4,6],"connectedNodes":[4,7]},{"coordinate":[4,7],"connectedNodes":[6]},{"coordinate":[2,6],"connectedNodes":[9,4]},{"coordinate":[2,7],"connectedNodes":[8]},{"coordinate":[2,4],"connectedNodes":[13,5,12]},{"coordinate":[4,4],"connectedNodes":[5,14,15]},{"coordinate":[2,3],"connectedNodes":[10]},{"coordinate":[1,4],"connectedNodes":[10]},{"coordinate":[5,4],"connectedNodes":[11]},{"coordinate":[4,3],"connectedNodes":[11,19]},{"coordinate":[3,3],"connectedNodes":[5,17]},{"coordinate":[3,2],"connectedNodes":[18,16,19,20]},{"coordinate":[2,2],"connectedNodes":[17]},{"coordinate":[4,2],"connectedNodes":[17,15,23,22]},{"coordinate":[3,1],"connectedNodes":[21,17,22]},{"coordinate":[2,1],"connectedNodes":[20]},{"coordinate":[4,1],"connectedNodes":[20,19]},{"coordinate":[5,2],"connectedNodes":[19]}]',
                                         pythonEnabled=False,
                                         theme=grass,
                                         threads=1,
                                         traffic_lights='[]',
                                         )

        TestCowCrashes.cow_level.save()

        blocks = Block.objects.filter(type__in=["move_forwards", "turn_left", "turn_right"])

        for block in blocks:
            new_block = LevelBlock(type=block, number=None, level=TestCowCrashes.cow_level)
            new_block.save()



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from itertools import chain

import permissions
from models import Block, LevelBlock, LevelDecor

from game.decor import get_decor_element
from game.theme import get_theme_by_pk
from game.character import get_character_by_pk


##########
# Levels #
##########

def get_loadable_levels(user):
    levels_owned_by_user = levels_owned_by(user)
    shared_levels = levels_shared_with(user)

    return levels_owned_by_user, shared_levels


def add_related_fields(levels):
    return levels.select_related("owner__user", "owner__teacher", "owner__student")


def levels_shared_with(user):
    if user.is_anonymous():
        return []

    shared_levels = user.shared

    return add_related_fields(shared_levels)


def levels_owned_by(user):
    if user.is_anonymous():
        return []

    levels = user.userprofile.levels

    return add_related_fields(levels)


def get_decor(level):
    """ Helper method parsing decor into a dictionary format 'sendable' to javascript. """
    decorData = []
    for ld in LevelDecor.objects.filter(level=level):
        decor = get_decor_element(name=ld.decorName, theme=level.theme)
        decorData.append({
            'x': int(ld.x),
            'y': int(ld.y),
            'z': int(decor.z_index),
            'decorName': str(ld.decorName),
            'width': int(decor.width),
            'height': int(decor.height),
            'url': str(decor.url),
        })

    return decorData


def set_decor(level, decor):
    set_decor_inner(level, decor, LevelDecor)


def set_decor_inner(level, decor, LevelDecor):
    """ Helper method creating LevelDecor objects given a list of decor in dictionary form."""
    LevelDecor.objects.filter(level=level).delete()

    level_decors = []
    for data in decor:
        level_decors.append(LevelDecor(
            level_id=level.id,
            x=data['x'],
            y=data['y'],
            decorName=data['decorName']
        ))
    LevelDecor.objects.bulk_create(level_decors)


def get_night_blocks(level):
    """ Helper method parsing blocks into a dictionary format 'sendable' to javascript. """
    coreBlockTypes = ['move_forwards', 'turn_left', 'turn_right', 'turn_around', 'controls_repeat',
                      'controls_repeat_until', 'controls_if', 'at_destination', 'road_exists', 'dead_end']
    coreNightBlocks = Block.objects.filter(type__in=coreBlockTypes)
    coreNightLevelBlocks = map(lambda block: LevelBlock(level=level, type=block), coreNightBlocks)
    existingLevelBlocks = LevelBlock.objects.filter(level=level)
    remainingBlocks = existingLevelBlocks.exclude(type__type__in=coreBlockTypes).order_by('type')
    levelBlocks = sorted(list(chain(coreNightLevelBlocks, remainingBlocks)), key=lambda levelBlock: levelBlock.type.id)

    return [{'type': lb.type.type, 'number': lb.number} for lb in levelBlocks]


def get_blocks(level):
    """ Helper method parsing blocks into a dictionary format 'sendable' to javascript. """
    levelBlocks = LevelBlock.objects.filter(level=level).order_by('type')
    return [{'type': lb.type.type, 'number': lb.number} for lb in levelBlocks]


def set_blocks(level, blocks):
    set_blocks_inner(level, blocks, LevelBlock, Block)


def set_blocks_inner(level, blocks, LevelBlock, Block):
    """ Helper method creating LevelBlock objects given a list of blocks in dictionary form."""
    LevelBlock.objects.filter(level=level).delete()

    level_blocks = []
    dictionary = blocks_dictionary(blocks, Block)

    for data in blocks:
        level_blocks.append(LevelBlock(
            level_id=level.id,
            type=dictionary[data['type']],
            number=data['number'] if 'number' in data else None
        ))
    LevelBlock.objects.bulk_create(level_blocks)


def blocks_dictionary(blocks, Block):
    types = map(lambda data: data['type'], blocks)
    block_objects = Block.objects.filter(type__in=types)
    result = {block.type: block for block in block_objects}
    return result


def save_level(level, data):
    level.name = data['name']
    level.path = data['path']
    level.origin = data['origin']
    level.destinations = data['destinations']
    level.max_fuel = data['max_fuel']
    level.traffic_lights = data['traffic_lights']
    level.cows = data['cows']
    level.blocklyEnabled = data.get('blocklyEnabled', True)
    level.pythonEnabled = data.get('pythonEnabled', False)
    level.pythonViewEnabled = data.get('pythonViewEnabled', False)
    level.theme = get_theme_by_pk(pk=data['theme'])
    level.character = get_character_by_pk(pk=data['character'])
    level.save()

    set_decor(level, data['decor'])
    set_blocks(level, data['blocks'])


def delete_level(level):
    level.delete()


def share_level(level, *users):
    level.shared_with.add(*users)


def unshare_level(level, *users):
    level.shared_with.remove(*users)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import csv

from django.http import HttpResponse
from django.utils.translation import ugettext_lazy

Single_Level_Header = [ugettext_lazy('Class'), ugettext_lazy('Name'), ugettext_lazy('Score'),
                       ugettext_lazy('Total Time'), ugettext_lazy('Start Time'), ugettext_lazy('Finish Time')]
Multiple_Levels_Header = [ugettext_lazy('Class'), ugettext_lazy('Name'), ugettext_lazy('Total Score'),
                          ugettext_lazy('Total Time'), ugettext_lazy('Started Levels %'),
                          ugettext_lazy('Attempted levels %'), ugettext_lazy('Finished levels %')]

def scoreboard_csv(student_data, requested_sorted_levels):
    if (len(requested_sorted_levels) > 1):
        return scoreboard_csv_multiple_levels(student_data, requested_sorted_levels)
    else:
        return scoreboard_csv_single_level(student_data)

def scoreboard_csv_multiple_levels(student_rows, levels):
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="scoreboard.csv"'

    header = header_for(levels)
    rows = map(to_array_multiple_levels, student_rows)

    writer = csv.writer(response)
    writer.writerow(header)
    writer.writerows(rows)

    return response

def scoreboard_csv_single_level(student_rows):
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="scoreboard.csv"'

    rows = map(to_array_single_level, student_rows)

    writer = csv.writer(response)
    writer.writerow(Single_Level_Header)
    writer.writerows(rows)

    return response

def header_for(levels):
    level_names = map(str, levels)
    return Multiple_Levels_Header + level_names

def to_array_multiple_levels(student_row):
    started, attempted, finished = student_row.progress
    result = [student_row.class_field.name, student_row.name, student_row.total_score, student_row.total_time,
              started, attempted, finished]

    return result + student_row.scores

def to_array_single_level(student_row):
    result = [student_row.class_field.name, student_row.name, student_row.total_score, student_row.total_time,
              student_row.start_time, student_row.finish_time]

    return result




# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import division
import game.messages as messages
import game.level_management as level_management

from django.shortcuts import render
from django.template import RequestContext
from django.utils.safestring import mark_safe
from django.db.models import Max
from game import random_road
from game.cache import cached_episode
from game.models import Attempt, Episode
from level_editor import play_anonymous_level
from django.core.cache import cache
from game import app_settings


def max_score(level):
    score = 0
    if not level.disable_route_score:
        score = score + 10
    if level.model_solution != "[]":
        score = score + 10
    return score


def fetch_episode_data_from_database(early_access):
    episode_data = []
    episode = Episode.objects.get(pk=1)
    while episode is not None:
        if episode.in_development and not early_access:
            break

        levels = []
        minName = None
        maxName = None
        for level in episode.levels:
            level_name = int(level.name)
            if not maxName or level_name > maxName:
                maxName = level_name
            if not minName or level_name < minName:
                minName = level_name

            levels.append({
                "id": level.id,
                "name": level_name,
                "maxScore": max_score(level),
                "title": get_level_title(level_name)})

        e = {"id": episode.id,
             "name": episode.name,
             "levels": levels,
             "first_level": minName,
             "last_level": maxName,
             "random_levels_enabled": episode.r_random_levels_enabled}

        episode_data.append(e)
        episode = episode.next_episode
    return episode_data


def fetch_episode_data(early_access):
    key = "episode_data"
    if early_access:
        key = "episode_data_early_access"
    data = cache.get(key)
    if data is None:
        data = fetch_episode_data_from_database(early_access)
        cache.set(key, data)
    return data


def get_level_title(i):
    title = 'title_level' + str(i)
    try:
        titleCall = getattr(messages, title)
        return mark_safe(titleCall())
    except AttributeError:
        return ""


def attach_attempts_to_level(attempts, level):
    level["score"] = attempts.get(level["id"])


def is_student(user):
    return hasattr(user.userprofile, 'student')


def levels(request):
    """ Loads a page with all levels listed.

    **Context**

    ``RequestContext``
    ``episodes``

    **Template:**

    :template:`game/level_selection.html`
    """
    user = request.user
    if user.is_authenticated() and is_student(user):
        best_attempts = Attempt.objects \
            .filter(student=user.userprofile.student) \
            .values("level_id") \
            .annotate(best_score=Max("score")) \
            .all()

        attempts = {a["level_id"]: a["best_score"] for a in best_attempts}
    else:
        attempts = {}

    def with_scores(level):
        attach_attempts_to_level(attempts, level)

    episode_data = fetch_episode_data(app_settings.EARLY_ACCESS_FUNCTION(request))
    for episode in episode_data:
        for level in episode["levels"]:
            with_scores(level)

    owned_level_data = []
    shared_level_data = []
    if not request.user.is_anonymous():
        owned_levels, shared_levels = level_management.get_loadable_levels(request.user)

        for level in owned_levels:
            owned_level_data.append({
                "id": level.id,
                "title": level.name,
                "score": attempts.get(level.id),
                "maxScore": 10
            })

        for level in shared_levels:
            shared_level_data.append({
                "id": level.id,
                "title": level.name,
                "owner": level.owner.user,
                "score": attempts.get(level.id),
                "maxScore": 10
            })

    context = RequestContext(request, {
        'episodeData': episode_data,
        'owned_levels': owned_level_data,
        'shared_levels': shared_level_data,
        'scores': attempts
    })
    return render(request, 'game/level_selection.html', context_instance=context)


def random_level_for_episode(request, episodeID):
    """ Generates a new random level based on the episodeID

    Redirects to :view:`game.views.play_level` with the id of the newly created :model:`game.Level`.
    """
    episode = cached_episode(episodeID)
    level = random_road.create(episode)
    return play_anonymous_level(request, level.id, False, random_level=True)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from django.db import migrations
from game.level_management import set_decor_inner, set_blocks_inner
import json

def new_level(apps, schema_editor):
    Level = apps.get_model('game', 'Level')
    Character = apps.get_model('game', 'Character')
    Theme = apps.get_model('game', 'Theme')
    LevelDecor = apps.get_model('game', 'LevelDecor')
    LevelBlock = apps.get_model('game', 'LevelBlock')
    Block = apps.get_model('game', 'Block')

    def set_decor(level, decor):
        set_decor_inner(level, decor, LevelDecor)

    def set_blocks(level, blocks):
        set_blocks_inner(level, blocks, LevelBlock, Block)

    level76 = Level(
        name='76',
        default=True,
        path='[{"coordinate":[2,6],"connectedNodes":[1]},{"coordinate":[3,6],"connectedNodes":[0,2]},{"coordinate":[4,6],"connectedNodes":[1,3,5]},{"coordinate":[5,6],"connectedNodes":[2,4]},{"coordinate":[5,5],"connectedNodes":[5,3,6]},{"coordinate":[4,5],"connectedNodes":[2,4,7]},{"coordinate":[5,4],"connectedNodes":[7,4,8]},{"coordinate":[4,4],"connectedNodes":[5,6,9]},{"coordinate":[5,3],"connectedNodes":[9,6,10]},{"coordinate":[4,3],"connectedNodes":[7,8,11]},{"coordinate":[5,2],"connectedNodes":[11,8,12]},{"coordinate":[4,2],"connectedNodes":[9,10,13]},{"coordinate":[5,1],"connectedNodes":[13,10,14,17]},{"coordinate":[4,1],"connectedNodes":[11,12,16]},{"coordinate":[6,1],"connectedNodes":[12,15]},{"coordinate":[7,1],"connectedNodes":[14]},{"coordinate":[4,0],"connectedNodes":[13,17]},{"coordinate":[5,0],"connectedNodes":[16,12]}]',
        traffic_lights='[]',
        destinations='[[7,1]]',
        origin='{"coordinate":[2,6],"direction":"E"}',
        max_fuel=50,
        blocklyEnabled=True,
        pythonEnabled=False,
        theme=Theme.objects.get(id=3),
        character=Character.objects.get(id='1'),
        model_solution='[9]',
        next_level=None,
        disable_route_score=True
    )
    level76.save()
    set_decor(level76, json.loads('[{"x":277,"y":382,"decorName":"tree2"},{"x":124,"y":381,"decorName":"tree2"},{"x":731,"y":547,"decorName":"pond"},{"x":639,"y":228,"decorName":"tree1"},{"x":227,"y":328,"decorName":"bush"},{"x":171,"y":191,"decorName":"pond"}]'))
    set_blocks(level76, json.loads('[{"type":"move_forwards","number":1},{"type":"turn_right","number":3},{"type":"wait"},{"type":"controls_repeat_while","number":1},{"type":"controls_if"},{"type":"logic_negate"},{"type":"at_destination"},{"type":"road_exists"}]'))

    level75 = Level.objects.get(name='75', default=1)
    level75.next_level = level76
    level75.model_solution = '[11]'
    level75.save()

class Migration(migrations.Migration):
    dependencies = [
        ('game', '0031_python_view'),
    ]

    operations = [
        migrations.RunPython(new_level)
    ]



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.

'''
    Character data
'''

from rest_framework.reverse import reverse


class Character(object):
    def __init__(self, pk, name, en_face, top_down, width, height):
        self.id = self.pk = pk
        self.name = name
        self.en_face = en_face
        self.top_down = top_down
        self.width = width
        self.height = height


CHARACTER_DATA = {
    'Van': Character(pk=1, name=u'Van', en_face=u'characters/front_view/Van.svg', top_down=u'characters/top_view/Van.svg', height='20', width='40'),
    'Dee': Character(pk=2, name=u'Dee', en_face=u'characters/front_view/Dee.svg', top_down=u'characters/top_view/Dee.svg', height='28', width='52'),
    'Nigel': Character(pk=3, name=u'Nigel', en_face=u'characters/front_view/Nigel.svg', top_down=u'characters/top_view/Nigel.svg', height='32', width='56'),
    'Kirsty': Character(pk=4, name=u'Kirsty', en_face=u'characters/front_view/Kirsty.svg', top_down=u'characters/top_view/Kirsty.svg', height='32', width='60'),
    'Wes': Character(pk=5, name=u'Wes', en_face=u'characters/front_view/Wes.svg', top_down=u'characters/top_view/Wes.svg', height='20', width='40'),
    'Phil': Character(pk=6, name=u'Phil', en_face=u'characters/front_view/Phil.svg', top_down=u'characters/top_view/Phil.svg', height='40', width='40'),
}


def get_character(name):
    """ Helper method to get a character."""
    return CHARACTER_DATA[name]


def get_all_character():
    return CHARACTER_DATA.values()


def get_character_by_pk(pk):
    for character in CHARACTER_DATA.values():
        if character.pk == int(pk):
            return character
    raise KeyError


def get_characters_url(pk, request):
    return reverse('character-detail', args={pk}, request=request)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from django.conf.urls import patterns, url, include
from django.views.decorators.cache import cache_page
from django.views.i18n import javascript_catalog

from django_js_reverse.views import urls_js

from game.views.api import level_list, level_detail, api_root, episode_list, episode_detail, levelblock_detail, \
    block_detail, theme_detail, block_list, theme_list, character_list, character_detail, level_for_episode, \
    levelblock_for_level, decor_list, decor_detail, map_for_level, leveldecor_detail, leveldecor_for_level, \
    mode_for_level, map_list
from game.views.level_editor import level_editor, delete_level_for_editor, load_level_for_editor, \
    save_level_for_editor, generate_random_map_for_editor, \
    get_sharing_information_for_editor, share_level_for_editor, play_anonymous_level, owned_levels, shared_levels
from game.views.level_moderation import level_moderation, get_students_for_level_moderation
from game.views.level_selection import levels, random_level_for_episode
from game.views.level import submit_attempt, play_default_level, start_episode, load_workspace, \
    load_list_of_workspaces, save_workspace, delete_workspace, \
    delete_level, play_custom_level, play_custom_level_from_editor, load_workspace_solution
from game.views.scoreboard import scoreboard
urlpatterns = patterns(
    '',
    url(r'^$', levels, name='levels'),
    url(r'^submit/$', submit_attempt, name='submit_attempt'),
    url(r'^(?P<levelName>[A-Z0-9]+)/$', play_default_level, name='play_default_level'),
    url(r'^custom/(?P<levelId>[0-9]+)/$', play_custom_level, name='play_custom_level'),
    url(r'^episode/(?P<episodeId>[0-9]+)/$', start_episode, name='start_episode'),
    url(r'^levels/random/([0-9]+)/$', random_level_for_episode, name='random_level_for_episode'),
    url(r'^scoreboard/$', scoreboard, name='scoreboard'),

    url(r'^workspace/', include([
        url(r'^load/(?P<workspaceID>[0-9]+)/$', load_workspace, name='load_workspace'),
        url(r'^load_list/$', load_list_of_workspaces, name='load_list_of_workspaces'),
        url(r'^save/$', save_workspace, name='save_workspace'),
        url(r'^save/(?P<workspaceID>[0-9]+)/$', save_workspace, name='save_workspace'),
        url(r'^delete/(?P<workspaceID>[0-9]+)/$', delete_workspace, name='delete_workspace'),
        url(r'^solution/(?P<levelName>[0-9]+)/$', load_workspace_solution, name='load_workspace_solution'),
    ])),

    url(r'^level_moderation/', include([
        url(r'^$', level_moderation, name='level_moderation'),
        url(r'^class/(?P<class_id>[0-9]+)/$', get_students_for_level_moderation, name='students_for_level_moderation'),
        url(r'^delete/(?P<levelID>[0-9]+)/$', delete_level, name='delete_level'),
    ])),

    url(r'^level_editor/', include([
        url(r'^$', level_editor, name='level_editor'),
        url(r'^play_anonymous/(?P<levelId>[0-9]+)/$', play_anonymous_level, name='play_anonymous_level'),
        url(r'^play_custom/(?P<levelId>[0-9]+)/$', play_custom_level_from_editor,
            name='play_custom_level_from_editor'),

        url(r'^get/(?P<levelID>[0-9]+)/$', load_level_for_editor, name='load_level_for_editor'),
        url(r'^delete/(?P<levelId>[0-9]+)/$', delete_level_for_editor, name='delete_level_for_editor'),
        url(r'^save/$', save_level_for_editor, name='save_level_for_editor'),
        url(r'^save/(?P<levelId>[0-9]+)/$', save_level_for_editor, name='save_level_for_editor'),
        url(r'^random/$', generate_random_map_for_editor, name='generate_random_map_for_editor'),

        url(r'^get_sharing_information/(?P<levelID>[0-9]+)/$',
            get_sharing_information_for_editor, name='get_sharing_information_for_editor'),
        url(r'^share/(?P<levelID>[0-9]+)/$',
            share_level_for_editor, name='share_level_for_editor'),

        url(r'^levels/owned/$', owned_levels, name='owned_levels'),
        url(r'^levels/shared/$', shared_levels, name='shared_levels'),
    ])),

    url(r'^api-auth/', include('rest_framework.urls', namespace='rest_framework')),
    url(r'^api/', include([
        url(r'^$', api_root),
        url(r'^blocks/$', block_list, name='block-list'),
        url(r'^blocks/(?P<pk>[0-9]+)/$', block_detail, name='block-detail'),
        url(r'^characters/$', character_list, name='character-list'),
        url(r'^characters/(?P<pk>[0-9]+)/$', character_detail, name='character-detail'),
        url(r'^decors/$', decor_list, name='decor-list'),
        url(r'^decors/(?P<pk>[0-9]+)/$', decor_detail, name='decor-detail'),
        url(r'^episodes/$', episode_list, name='episode-list'),
        url(r'^episodes/(?P<pk>[0-9]+)/$', episode_detail, name='episode-detail'),
        url(r'^episodes/(?P<pk>[0-9]+)/levels/$', level_for_episode, name='level-for-episode'),
        url(r'^levels/$', level_list, name='level-list'),
        url(r'^levels/(?P<pk>[0-9]+)/$', level_detail, name='level-detail'),
        url(r'^levels/(?P<pk>[0-9]+)/blocks/$', levelblock_for_level, name='levelblock-for-level'),
        url(r'^levels/(?P<pk>[0-9]+)/decors/$', leveldecor_for_level, name='leveldecor-for-level'),
        url(r'^levels/(?P<pk>[0-9]+)/map/$', map_for_level, name='map-for-level'),
        url(r'^levels/(?P<pk>[0-9]+)/mode/$', mode_for_level, name='mode-for-level'),
        url(r'^levelblocks/(?P<pk>[0-9]+)/$', levelblock_detail, name='levelblock-detail'),
        url(r'^leveldecors/(?P<pk>[0-9]+)/$', leveldecor_detail, name='leveldecor-detail'),
        url(r'^maps/$', map_list, name='map-list'),
        url(r'^themes/$', theme_list, name='theme-list'),
        url(r'^themes/(?P<pk>[0-9]+)/$', theme_detail, name='theme-detail'),
    ])),

    url(r'^js-reverse/$', cache_page(60*60*24)(urls_js), name='js-reverse'),

    url(r'^js-i18n/$', cache_page(60*60*24)(javascript_catalog), {'packages': ('game',), }, name='rapid-router/javascript-catalog'),
)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import json

from django.core.urlresolvers import reverse
from django.test.testcases import TestCase
from django.test.client import Client

from hamcrest import *

from game.tests.utils.teacher import signup_teacher_directly, create_school, add_teacher_to_school
from game.tests.utils.level import create_save_level


class LevelEditorTestCase(TestCase):

    def setUp(self):
        self.client = Client()

    def login(self, email, password):
        self.client.post(reverse('teach'), {'login-email': email, 'login-password': password, 'login': ''}, follow=True)

    def get_sharing_information(self, level_id):
        url = reverse('get_sharing_information_for_editor', args=[level_id])
        response = self.client.get(url)
        return response

    def test_level_sharing_with_no_school(self):
        teacher1, email1, password1 = signup_teacher_directly()
        teacher2, _, _ = signup_teacher_directly()

        self.login(email1, password1)
        level_id = create_save_level(teacher1)

        sharing_info1 = json.loads(self.get_sharing_information(level_id).getvalue())
        assert_that(len(sharing_info1['teachers']), equal_to(0))

    def test_level_sharing_with_school(self):
        teacher1, email1, password1 = signup_teacher_directly()
        teacher2, _, _ = signup_teacher_directly()

        self.login(email1, password1)
        level_id = create_save_level(teacher1)

        school1 = create_school()
        add_teacher_to_school(teacher1, school1)
        add_teacher_to_school(teacher2, school1)

        sharing_info1 = json.loads(self.get_sharing_information(level_id).getvalue())
        assert_that(len(sharing_info1['teachers']), equal_to(1))

    def test_level_sharing_with_empty_school(self):
        teacher1, email1, password1 = signup_teacher_directly()
        teacher2, _, _ = signup_teacher_directly()

        self.login(email1, password1)
        level_id = create_save_level(teacher1)

        school1 = create_school()
        add_teacher_to_school(teacher1, school1)

        sharing_info1 = json.loads(self.get_sharing_information(level_id).getvalue())
        assert_that(len(sharing_info1['teachers']), equal_to(0))



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2016, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from datetime import timedelta, datetime
from django.utils.timezone import utc

from django.test import TestCase
from hamcrest import *

from game.models import Level, Attempt
from game.views.scoreboard import StudentRow, scoreboard_data
from game.views.scoreboard_csv import scoreboard_csv_multiple_levels, scoreboard_csv_single_level
from portal.models import Class
from portal.tests.utils.classes import create_class_directly
from portal.tests.utils.student import create_school_student_directly
from portal.tests.utils.teacher import signup_teacher_directly

Headers = ['Class', 'Name', 'Total Score', 'Total Time', 'Started Levels %', 'Attempted levels %', 'Finished levels %']


class ScoreboardTestCase(TestCase):

    def test_teacher_multiple_students_multiple_levels(self):
        level_ids = ids_of_levels_named(["1", "2"])
        level1 = Level.objects.get(name="1")
        level2 = Level.objects.get(name="2")

        clas, student, student2 = set_up_data()

        create_attempt(student, level1, 10.5)
        create_attempt(student2, level1, 2.3)
        create_attempt(student2, level2, 16.7)

        student_data, headers = scoreboard_data(Teacher(), level_ids, [clas.id])

        assert_that(headers, equal_to(['Class', 'Name', 'Total Score', 'Total Time', 'Progress', u'Level 1', u'Level 2']))
        assert_that(student_data, has_length(2))

        assert_student_row(student_row=student_data[0],
                           class_name=clas.name,
                           student_name=student.user.user.first_name,
                           total_score=10.5,
                           total_time=timedelta(0),
                           progress=(0.0, 0.0, 50.0),
                           scores=[10.5, ''])

        assert_student_row(student_row=student_data[1],
                           class_name=clas.name,
                           student_name=student2.user.user.first_name,
                           total_score=19.0,
                           total_time=timedelta(0),
                           progress=(0.0, 50.0, 50.0),
                           scores=[2.3, 16.7])

    def test_teacher_multiple_students_single_level(self):
        level_ids = ids_of_levels_named(["1"])
        level1 = Level.objects.get(name="1")

        clas, student, student2 = set_up_data()

        create_attempt(student, level1, 10.5)
        create_attempt(student2, level1, 2.3)

        student_data, headers = scoreboard_data(Teacher(), level_ids, [clas.id])

        assert_that(headers, equal_to(['Class', 'Name', 'Score', 'Total Time', 'Start Time', 'Finish Time']))
        assert_that(student_data, has_length(2))

        assert_student_row_single_level(student_row=student_data[0],
                                        class_name=clas.name,
                                        student_name=student.user.user.first_name,
                                        total_score=10.5,
                                        total_time=timedelta(0))

        assert_student_row_single_level(student_row=student_data[1],
                                        class_name=clas.name,
                                        student_name=student2.user.user.first_name,
                                        total_score=2.3,
                                        total_time=timedelta(0))

    def test_student_multiple_students_multiple_levels(self):
        level_ids = ids_of_levels_named(["1", "2"])
        level1 = Level.objects.get(name="1")
        level2 = Level.objects.get(name="2")

        clas, student, student2 = set_up_data(True)

        create_attempt(student, level1, 10.5)
        create_attempt(student2, level1, 2.3)
        create_attempt(student2, level2, 16.7)

        student_data, headers = scoreboard_data(Student(student), level_ids, [clas.id])

        assert_that(headers, equal_to(['Class', 'Name', 'Total Score', 'Total Time', 'Progress', u'Level 1', u'Level 2']))
        assert_that(student_data, has_length(2))

        assert_student_row(student_row=student_data[0],
                           class_name=clas.name,
                           student_name=student.user.user.first_name,
                           total_score=10.5,
                           total_time=timedelta(0),
                           progress=(0.0, 0.0, 50.0),
                           scores=[10.5, ''])

        assert_student_row(student_row=student_data[1],
                           class_name=clas.name,
                           student_name=student2.user.user.first_name,
                           total_score=19.0,
                           total_time=timedelta(0),
                           progress=(0.0, 50.0, 50.0),
                           scores=[2.3, 16.7])

    def test_student_multiple_students_single_level(self):
        level_ids = ids_of_levels_named(["2"])
        level2 = Level.objects.get(name="2")

        clas, student, student2 = set_up_data(True)

        create_attempt(student, level2, 10.5)
        create_attempt(student2, level2, 16.7)

        student_data, headers = scoreboard_data(Student(student), level_ids, [clas.id])

        assert_that(headers, equal_to(['Class', 'Name', 'Score', 'Total Time', 'Start Time', 'Finish Time']))
        assert_that(student_data, has_length(2))

        assert_student_row_single_level(student_row=student_data[0],
                                        class_name=clas.name,
                                        student_name=student.user.user.first_name,
                                        total_score=10.5,
                                        total_time=timedelta(0))

        assert_student_row_single_level(student_row=student_data[1],
                                        class_name=clas.name,
                                        student_name=student2.user.user.first_name,
                                        total_score=16.7,
                                        total_time=timedelta(0))

    def test_student_multiple_students_multiple_levels_cannot_see_classmates(self):
        level_ids = ids_of_levels_named(["1", "2"])
        level1 = Level.objects.get(name="1")
        level2 = Level.objects.get(name="2")

        clas, student, student2 = set_up_data()
        create_attempt(student, level1, 10.5)
        create_attempt(student2, level1, 2.3)
        create_attempt(student2, level2, 16.7)

        student_data, headers = scoreboard_data(Student(student), level_ids, [clas.id])

        assert_that(headers, equal_to(['Class', 'Name', 'Total Score', 'Total Time', 'Progress', u'Level 1', u'Level 2']))
        assert_that(student_data, has_length(1))

        assert_student_row(student_row=student_data[0],
                           class_name=clas.name,
                           student_name=student.user.user.first_name,
                           total_score=10.5,
                           total_time=timedelta(0),
                           progress=(0.0, 0.0, 50.0),
                           scores=[10.5, ''])


class ScoreboardCsvTestCase(TestCase):
    def test_multiple_levels(self):
        levels = Level.objects.sorted_levels()
        student_rows = [(self.student_row()), (self.student_row())]

        response = scoreboard_csv_multiple_levels(student_rows, levels)

        actual_header, actual_rows = self.actual_data(response.content)

        expected_header = self.expected_header(levels)
        expected_rows = self.expected_rows_multiple_levels(student_rows)

        assert_that(actual_header, equal_to(expected_header))

        assert_that(actual_rows, equal_to(expected_rows))

    def test_single_level(self):
        student_rows = [(self.student_row()), (self.student_row())]
        response = scoreboard_csv_single_level(student_rows)

        actual_header, actual_rows = self.actual_data(response.content)

        expected_header = 'Class,Name,Score,Total Time,Start Time,Finish Time'
        expected_rows = self.expected_rows_single_level(student_rows)

        assert_that(actual_header, equal_to(expected_header))

        assert_that(actual_rows, equal_to(expected_rows))

    def expected_rows_single_level(self, student_rows):
        return map(self.expected_row_single_level, student_rows) + [""]

    def expected_rows_multiple_levels(self, student_rows):
        return map(self.expected_row_multiple_levels, student_rows) + [""]

    def student_row(self):
        email, password = signup_teacher_directly()
        _, class_name, access_code = create_class_directly(email)
        _, _, student = create_school_student_directly(access_code)

        total_time = timedelta(0, 30)
        scores = [x for x in range(20)]
        total_score = sum(scores)
        progress = (0, 0, 0)

        all_scores = scores + [""] * 89

        row = StudentRow(student=student,
                         total_time=total_time,
                         total_score=total_score,
                         start_time=datetime.fromtimestamp(1435305072, tz=utc),
                         finish_time=datetime.fromtimestamp(1438305072, tz=utc),
                         progress=progress,
                         scores=all_scores,
                         class_field=Class(name="MyClass"))
        return row

    def expected_row_multiple_levels(self, student_row):
        beginning = "%s,%s,190,0:00:30,0,0,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19," \
                    % (student_row.class_field.name, student_row.name)
        padding = ','.join([""] * 89)
        return beginning + padding

    def expected_row_single_level(self, student_row):
        return "%s,%s,190,0:00:30,2015-06-26 07:51:12+00:00,2015-07-31 01:11:12+00:00" % (
            student_row.class_field.name, student_row.name)

    def expected_header(self, levels):
        level_strings = map(str, levels)
        all_header_strings = Headers + level_strings
        joined = ','.join(all_header_strings)
        return joined

    def actual_data(self, content):
        split = content.split("\r\n")
        header = split[0]
        rows = split[1:]
        return header, rows


class Student:
    def __init__(self, student):
        self.student = student

    def is_student(self): return True

    def is_teacher(self): return False

    def is_independent_student(self): return False


class Teacher:
    def is_student(self): return False

    def is_teacher(self): return True

    def is_independent_student(self): return False


def assert_student_row(student_row, class_name, student_name, total_score, total_time, progress, scores):
    assert_that(student_row.class_field.name, equal_to(class_name))
    assert_that(student_row.name, equal_to(student_name))
    assert_that(student_row.total_score, equal_to(total_score))
    assert_that(student_row.total_time, equal_to(total_time))
    assert_that(student_row.progress, equal_to(progress))
    assert_that(student_row.scores, equal_to(scores))


def assert_student_row_single_level(student_row, class_name, student_name, total_score, total_time):
    assert_that(student_row.class_field.name, equal_to(class_name))
    assert_that(student_row.name, equal_to(student_name))
    assert_that(student_row.total_score, equal_to(total_score))
    assert_that(student_row.total_time, equal_to(total_time))


def create_attempt(student, level, score):
    attempt = Attempt.objects.create(finish_time=datetime.fromtimestamp(1435305072),
                                     level=level,
                                     student=student,
                                     score=score,
                                     is_best_attempt=True)
    attempt.start_time=datetime.fromtimestamp(1435305072)
    attempt.save()

def ids_of_levels_named(names):
    levels = Level.objects.filter(name__in=names)
    assert_that(len(levels), equal_to(len(names)))
    level_ids = map(lambda x: x.id, levels)
    return level_ids


def set_up_data(classmates_data_viewable=False):
    email, password = signup_teacher_directly()
    clas, class_name, access_code = create_class_directly(email)
    if classmates_data_viewable:
        clas.classmates_data_viewable = True
        clas.save()
    _, _, student = create_school_student_directly(access_code)
    _, _, student2 = create_school_student_directly(access_code)
    create_random_school_data()
    return clas, student, student2


def create_random_school_data():
    email, password = signup_teacher_directly()
    clas, class_name, access_code = create_class_directly(email)
    create_school_student_directly(access_code)
    create_school_student_directly(access_code)
